#include <statemachine.h>

// see https://github.com/adafruit/Adafruit_NeoPixel?tab=readme-ov-file
// and https://adafruit.github.io/Adafruit_NeoPixel/html/index.html 
//
#include <Adafruit_NeoPixel.h>
#define PIN_WS2812B  2   // The Arduino Nano pin that connects to WS2812B
#define NUM_PIXELS   64  // The number of LEDs (pixels) on WS2812B
Adafruit_NeoPixel WS2812B(NUM_PIXELS, PIN_WS2812B, NEO_GRB + NEO_KHZ800);

#define TIMESTEP_STATEMACHINE 300

// check the Color attribute entry in
// https://adafruit.github.io/Adafruit_NeoPixel/html/class_adafruit___neo_pixel.html
//
uint32_t TRAFFIC_LIGHT_RED       = WS2812B.Color(255,   0,   0);
uint32_t TRAFFIC_LIGHT_YELLOW    = WS2812B.Color(200, 200,   0);
uint32_t TRAFFIC_LIGHT_GREEN     = WS2812B.Color(  0, 255,   0);
uint32_t TRAFFIC_LIGHT_OFF       = WS2812B.Color(255,   0,   0);

extern unsigned char current_state;
extern unsigned char ped_request;

void ShowState(unsigned char);

unsigned char PSTATES=0;
unsigned char performNextStep=0;        // flag to indicate "start the next step in our statemachine

const int INTERRUPT_PIN = 2;

void setup() {
  WS2812B.begin(); // INITIALIZE WS2812B strip object (REQUIRED)
  ped_request = false;
  //attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN), myISR, RISING); // auf die fallende Flanke reaieren
    
  Serial.begin(115200);
}

void loop() {
    ShowState(current_state);
    delay(TIMESTEP_STATEMACHINE);
    Do_Step('x');
}


void Ampel1_rot()       { WS2812B.setPixelColor(10, TRAFFIC_LIGHT_RED);
                          WS2812B.setPixelColor(18, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(26, TRAFFIC_LIGHT_OFF);
                        }
void Ampel1_rotgelb()   { WS2812B.setPixelColor(10, TRAFFIC_LIGHT_RED);
                          WS2812B.setPixelColor(18, TRAFFIC_LIGHT_YELLOW);
                          WS2812B.setPixelColor(26, TRAFFIC_LIGHT_OFF);
                        }
void Ampel1_gelb()      { WS2812B.setPixelColor(10, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(18, TRAFFIC_LIGHT_YELLOW);
                          WS2812B.setPixelColor(26, TRAFFIC_LIGHT_OFF);
                        }
void Ampel1_gruen()     { WS2812B.setPixelColor(10, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(18, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(26, TRAFFIC_LIGHT_GREEN);
                        }



void Ampel2_rot()       { WS2812B.setPixelColor(36, TRAFFIC_LIGHT_RED);
                          WS2812B.setPixelColor(44, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(52, TRAFFIC_LIGHT_OFF);
                        }
void Ampel2_rotgelb()   { WS2812B.setPixelColor(36, TRAFFIC_LIGHT_RED);
                          WS2812B.setPixelColor(44, TRAFFIC_LIGHT_YELLOW);
                          WS2812B.setPixelColor(52, TRAFFIC_LIGHT_OFF);
                        }
void Ampel2_gelb()      { WS2812B.setPixelColor(36, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(44, TRAFFIC_LIGHT_YELLOW);
                          WS2812B.setPixelColor(52, TRAFFIC_LIGHT_OFF);
                        }
void Ampel2_gruen()     { WS2812B.setPixelColor(36, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(44, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(52, TRAFFIC_LIGHT_GREEN);
                        }

void FussgAmpel_rot()   { WS2812B.setPixelColor(22, TRAFFIC_LIGHT_RED);
                          WS2812B.setPixelColor(30, TRAFFIC_LIGHT_OFF);
                        }
void FussgAmpel_gruen() { WS2812B.setPixelColor(22, TRAFFIC_LIGHT_OFF);
                          WS2812B.setPixelColor(30, TRAFFIC_LIGHT_GREEN);
                        }


/*
 * Zustand der Kreuzung anzeigen; an die Dioden "ausgeben"
 */
void ShowState(unsigned char state)
{
  WS2812B.clear();
  switch(state)
  {
    case ALLES_STOP1    : Ampel1_rot();     Ampel2_rot();     FussgAmpel_rot();   break;
    case Horiz_RotGelb  : Ampel1_rotgelb(); Ampel2_rot();     FussgAmpel_rot();   break;
    case Horiz_Gruen    : Ampel1_gruen();   Ampel2_rot();     FussgAmpel_rot();   break;
    case Horiz_Gelb     : Ampel1_gelb();    Ampel2_rot();     FussgAmpel_rot();   break;
    case ALLES_STOP2    : Ampel1_rot();     Ampel2_rot();     FussgAmpel_rot();   break;
    case Vert_RotGelb   : Ampel1_rot();     Ampel2_rotgelb(); FussgAmpel_rot();   break;
    case Vert_Leer      : Ampel1_rot();     Ampel2_gruen();   FussgAmpel_rot();   break;
    case Vert_Gelb      : Ampel1_rot();     Ampel2_gelb();    FussgAmpel_rot();   break;
    case Fussg_Gruen1   : Ampel1_rot();     Ampel2_rot();     FussgAmpel_gruen(); break;
    case Fussg_Gruen2   : Ampel1_rot();     Ampel2_rot();     FussgAmpel_gruen(); break;
    default             : break;
  }
  WS2812B.show();
}

/*
 * Landepunkt für die Ampel-Anforderungstaste
 */
void myISR() {
  noInterrupts();
  delay(1000);
  Serial.write("button pressed\n");
  if(ped_request==false)
  {
    ped_request = true;
  }
  interrupts();
}
